﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Globalization;

namespace InterleaveSystem24
{
    class EntryPoint
    {
        static void Main(string[] args)
        {
            // Attempt to obtain the command line arguments
            if (args.Length < 3)
            {
                Console.WriteLine(@"Converts Sega System 24 disk images from linear raw sector dumps, as used in");
                Console.WriteLine(@"MAME, to the interleaved "".s24"" sector format accepted by SAMdisk 3.8.10.");
                Console.WriteLine();
                Console.WriteLine(@"Usage: InterleaveSystem24.exe <infile> <outfile> <trackSize>");
                Console.WriteLine(@"infile     The System 24 disk image");
                Console.WriteLine(@"outfile    The interleaved .s24 disk image");
                Console.WriteLine(@"trackSize  The byte length of a track. 2F00 for hotrod, 2D00 for others.");
                Console.WriteLine();
                Console.WriteLine(@"Example: InterleaveSystem24.exe ""ds3-5000-04d.img"" ""ds3-5000-04d.s24"" 2D00");
                return;
            }
            string inFilePath = args[0];
            string outFilePath = args[1];
            string trackSizeAsString = args[2];

            // Attempt to convert the track size to a number
            int trackSize;
            if (!Int32.TryParse(trackSizeAsString, NumberStyles.HexNumber, null, out trackSize))
            {
                Console.WriteLine("Failed to convert the input sector size \"{0}\" to an integer.", trackSizeAsString);
                return;
            }

            // Attempt to read the sector data from the input file
            Console.WriteLine("Reading input data from file \"{0}\".", inFilePath);
            byte[] inFileData;
            try
            {
                inFileData = File.ReadAllBytes(inFilePath);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception when opening file at path \"{0}\".", inFilePath);
                Console.WriteLine(ex.ToString());
                return;
            }

            // Interleave the track data into a form supported by SAMdisk 3.8.10. The current code in SAMdisk expects
            // the complete data for the first head to be listed first in the file for all sectors, followed by all the
            // data for the second head. This is contrary to the image format used in MAME, and the way the media is
            // used by the system itself, which store data sequentially alternating between each head.
            Console.WriteLine("Converting sector data.");
            int trackCount = inFileData.Length / trackSize;
            byte[] outputData = new byte[trackCount * trackSize];
            int outPos = 0;
            for (int i = 0; i < trackCount; i += 2)
            {
                Array.Copy(inFileData, i * trackSize, outputData, outPos, trackSize);
                outPos += trackSize;
            }
            for (int i = 1; i < trackCount; i += 2)
            {
                Array.Copy(inFileData, i * trackSize, outputData, outPos, trackSize);
                outPos += trackSize;
            }

            // Attempt to write the converted sector data to the output file
            Console.WriteLine("Writing output data to file \"{0}\".", outFilePath);
            try
            {
                File.WriteAllBytes(outFilePath, outputData);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception when writing to output file at path \"{0}\".", outFilePath);
                Console.WriteLine(ex.ToString());
                return;
            }

            // Report success, and exit the program.
            Console.WriteLine("Conversion complete.");
        }
    }
}
